@@ -57,6 +57,8 @@
 ;; Server runtime vars
 (defvar telega-server--buffer nil)
 (defvar telega-server--extra 0 "Value for :@extra used by `telega-server--call'.")
+(defvar telega-server--callback-extra nil
+  "Bound to extra value when callback is called.")
 (defvar telega-server--callbacks nil "Callbacks ruled by extra")
 (defvar telega-server--results nil)
 (defvar telega-server--on-event-func #'telega--on-event
@@ -121,6 +123,13 @@ Otherwise query user about building flags."
   (telega-test-env 'quiet)
   (when (or build-flags
             (y-or-n-p "Build `telega-server'? "))
+    ;; NOTE: check header files exists before running compilation
+    (unless (file-exists-p (expand-file-name "include/td/telegram/td_json_client.h"
+                                             telega-server-libs-prefix))
+      (user-error "TDLib is not installed into \"%s\". \
+Set `telega-server-libs-prefix' to the TDLib installion path"
+                  telega-server-libs-prefix))
+    (message "Telega: building telega-server...")
     (let ((default-directory telega--lib-directory))
       (unless build-flags
         (setq build-flags
@@ -141,13 +150,19 @@ Otherwise query user about building flags."
                         "LIBS_PREFIX=" (expand-file-name telega-server-libs-prefix) " "
                         "INSTALL_PREFIX=" (expand-file-name telega-directory) " "
                         "server-reinstall")))
-        (error "`telega-server' installation failed")))))
+        (error "`telega-server' installation failed"))
+      (message "Telega: building telega-server...DONE"))))
 
 (defun telega-server--ensure-build ()
   "Make sure telega-server is build and can run."
   (if telega-use-docker
-      (or (executable-find "docker")
-          (error "`docker' not found in exec-path"))
+      (or (executable-find (if (stringp telega-use-docker)
+                               telega-use-docker
+                             "docker"))
+          (error "`%s' not found in exec-path"
+                 (if (stringp telega-use-docker)
+                     telega-use-docker
+                   "docker")))
 
     (let ((exec-path (cons telega-directory exec-path)))
       (or (if (executable-find telega-server-command)
@@ -232,16 +247,25 @@ Return parsed command."
             (cl-assert (= (following-char) ?\n))
             (delete-char 1)))))))
 
+(defun telega-tl-error-equal (tl-error code &optional message)
+  "Return non-nil if TL-ERROR has CODE and MESSAGE."
+  (and (equal (plist-get tl-error :code) code)
+       (or (null message)
+           (equal (plist-get tl-error :message) message))))
+
 (defvar telega-server--last-error)
 (defsubst telega-server--dispatch-cmd (cmd value)
   "Dispatch command CMD."
   (telega-debug "%s %s: %S" cmd (propertize "IN" 'face 'bold) value)
 
   (cond ((string= cmd "event")
-         (let* ((extra (plist-get value :@extra))
-                (call-cb (telega-server--callback-get extra)))
+         (let* ((telega-server--callback-extra (plist-get value :@extra))
+                (call-cb (telega-server--callback-get
+                          telega-server--callback-extra)))
+           ;; NOTE: Some callbacks might stuck in the
+           ;; `telega-server--callbacks' due to runtime errors causing
            (if call-cb
-               (telega-server--callback-rm extra)
+               (telega-server--callback-rm telega-server--callback-extra)
              (setq call-cb telega-server--on-event-func))
 
            ;; Function call may return errors
@@ -249,9 +273,19 @@ Return parsed command."
                    ;; If the error code is 406, the error message must
                    ;; not be processed in any way and must not be
                    ;; displayed to the user
-                   (= (plist-get value :code) 406)
+                   (telega-tl-error-equal value 406)
                    ;; 404 - webpage or message not found
-                   (= (plist-get value :code) 404)
+                   (telega-tl-error-equal value 404)
+
+                   ;; 400 - Special case for `downloadFile' to alsways
+                   ;; call it's callback in order to delete update
+                   ;; callback
+                   (telega-tl-error-equal
+                    value 400
+                    "File download has failed or was canceled") ;XXX
+
+                   ;; For `joinChatByInviteLink'
+                   (telega-tl-error-equal value 400 "INVITE_REQUEST_SENT")
                    )
                (funcall call-cb value)
 
@@ -268,7 +302,6 @@ Return parsed command."
          (telega--on-error value))
 
         (t
-         (telega-debug "%s %s: %S" (propertize "IN" 'face 'bold) cmd value)
          (error "Unknown cmd from telega-server: %s" cmd))))
 
 (defun telega-server--idle-timer-function ()
@@ -327,8 +360,15 @@ Used to optimize events processing in the `telega-server--parse-commands'."
            (updateFile
             (eq (telega--tl-get cmd1-value :file :id)
                 (telega--tl-get cmd2-value :file :id)))
-           ((updateChatLastMessage
-             updateChatReadInbox
+           (updateChatLastMessage
+            ;; NOTE: Collapse only if `:last_message' is provided in
+            ;; both events, so history gaps won't collapse into
+            ;; consequent `updateChatLastMessage'
+            (and (eq (plist-get cmd1-value :chat_id)
+                     (plist-get cmd2-value :chat_id))
+                 (plist-get cmd1-value :last_message)
+                 (plist-get cmd2-value :last_message)))
+           ((updateChatReadInbox
              updateChatReadOutbox
              updateChatUnreadMentionCount
              updateChatOnlineMemberCount
@@ -354,7 +394,7 @@ Used to optimize events processing in the `telega-server--parse-commands'."
                    (plist-get cmd2-value :chat_list)))
            )
          (progn
-           (telega-debug "Collapsed events: %S" value-type)
+           (telega-debug "Collapsing: %S into %S" cmd2-value cmd1-value)
            t)
          )))
 
@@ -364,13 +404,20 @@ Used to optimize events processing in the `telega-server--parse-commands'."
   (let (cmd-val parsed-commands)
     ;; NOTE: First parse all commands, then optimize events, because
     ;; some events (such as `updateFile', `updateChatLastMessage',
-    ;; etc) can be collapsed to single event, then dispatch all the
-    ;; events left after optimization
+    ;; etc) can be collapsed to a single event (last one), then dispatch
+    ;; all the events left after optimization
     (while (setq cmd-val (telega-server--parse-cmd))
-      (setq parsed-commands (cons cmd-val parsed-commands)))
-    (dolist (cmd (cl-delete-duplicates (nreverse parsed-commands)
-                                       :test #'telega-server--commands-equal))
-      (apply #'telega-server--dispatch-cmd cmd))
+      (setq parsed-commands
+            (cons cmd-val
+                  (cl-delete cmd-val parsed-commands
+                             :test #'telega-server--commands-equal))))
+    (dolist (cmd (nreverse parsed-commands))
+      ;; NOTE: make sure all commands are processed even if some
+      ;; triggers an error
+      (condition-case-unless-debug nil
+          (apply #'telega-server--dispatch-cmd cmd)
+        (error
+         (message "telega: Error while processing cmd %S" cmd))))
 
     (if telega-server--idle-timer
         (timer-set-time telega-server--idle-timer
@@ -449,7 +496,8 @@ when result is received.
 If CALLBACK is specified return `:@extra' value used for the call.
 COMMAND is passed directly to `telega-server--send'."
   (unless (plist-get sexp :@extra)
-    (setq sexp (plist-put sexp :@extra (cl-incf telega-server--extra))))
+    ;; Inject `:@extra' without modifying `sexp'
+    (setq sexp (nconc (list :@extra (cl-incf telega-server--extra)) sexp)))
   (telega-server--send sexp command)
 
   (if callback
@@ -475,7 +523,7 @@ COMMAND is passed directly to `telega-server--send'."
         (remhash cb-extra telega-server--results)
 
         (when (and (not ret) telega-server--last-error)
-          (user-error
+          (error
            "telega error=%d: %s"
            (plist-get telega-server--last-error :code)
            (plist-get telega-server--last-error :message)))
