@@ -32,6 +32,7 @@
 (require 'telega-tdlib)
 (require 'telega-i18n)
 (require 'telega-sticker)
+(require 'telega-util)
 
 ;; telega-chat.el depends on telega-tme.el
 (declare-function telega-chat-get "telega-chat" (chat-id &optional offline-p))
@@ -40,15 +41,16 @@
 (declare-function telega-chat--pop-to-buffer "telega-chat" (chat &optional no-history-load))
 (declare-function telega-chatbuf--prompt-update "telega-chat")
 
+(declare-function telega-webpage--instant-view "telega-webpage" (url &optional sitename instant-view))
+(declare-function telega-browse-url "telega-webpage" (url &optional in-web-browser))
+
 
 (defun telega-tme--media-timestamp-callback (media-timestamp-str)
   "Generate callback to open media message at MEDIA-TIMESTAMP."
   (when media-timestamp-str
     (lambda ()
       (let ((msg (telega-msg-at (point))))
-        (when (telega-msg-type-p
-               '(messageVideoNote messageVoiceNote messageAudio messageVideo)
-               msg)
+        (when (telega-msg-match-p msg '(type VideoNote VoiceNote Audio Video))
           (let ((telega-ffplay-media-timestamp
                  (string-to-number media-timestamp-str)))
             (telega-msg-open-content msg)))))))
@@ -67,7 +69,8 @@ PARAMS is a plist of additional parameters to the returned link."
             (when params
               (concat "?" (telega-tme-build-query-string params)))
             (when-let ((msg (unless (telega-chat-p chat-or-msg) chat-or-msg)))
-              (concat "#" (number-to-string (/ (plist-get msg :id) 1048576))))
+              (concat "#" (number-to-string
+                           (/ (plist-get msg :id) telega-msg-id-step))))
             )))
 
 (defun telega-tme-open-internal (chat-spec &optional post-spec params)
@@ -83,9 +86,9 @@ PARAMS is a plist with additional parameters, supported parameters are:
          (post-id (when post-spec (string-to-number post-spec)))
          ;; NOTE: For backward compatibility check if POST-ID is
          ;; actually a MSG-ID
-         (msg-id (when post-id (if (zerop (% post-id 1048576))
+         (msg-id (when post-id (if (zerop (% post-id telega-msg-id-step))
                                    post-id
-                                 (* post-id 1048576)))))
+                                 (* post-id telega-msg-id-step)))))
     (cond ((plist-get params :open_content)
            (cl-assert msg-id)
            (telega-msg-get chat msg-id
@@ -100,13 +103,14 @@ PARAMS is a plist with additional parameters, supported parameters are:
   "Convert POST number to the message id."
   ;; See https://github.com/tdlib/td/issues/16
   ;; msg-id = post * 1048576
-  (* (string-to-number post) 1048576))
+  (* (string-to-number post) telega-msg-id-step))
 
 (defun telega-tme-open-privatepost (supergroup post &optional media-timestamp)
   "Open POST in private SUPERGROUP."
-  (when-let ((chat (telega-chat-get
-                    (string-to-number (concat "-100" supergroup))
-                    'offline)))
+  (let ((chat (telega-chat-get (string-to-number (concat "-100" supergroup))
+                               'offline)))
+    (unless chat
+      (error "telega: %s" (telega-i18n "lng_error_post_link_invalid")))
     (telega-chat--goto-msg chat (telega-tme--post-msg-id post) 'highlight
       (telega-tme--media-timestamp-callback media-timestamp))))
 
@@ -120,22 +124,20 @@ PARAMS are additional params."
         ;; See https://core.telegram.org/bots#deep-linking for
         ;; `:start' and `:startgroup' meaning
         ((plist-get params :startgroup)
-         (let ((bot-user (telega-chat-user
-                          (telega--searchPublicChat username) 'inc-bots))
+         (let ((bot-user (telega-chat-user (telega--searchPublicChat username)))
                (chat (telega-completing-read-chat
                       ;; TODO: i18n
                       (format "Start «%s» in group: "
                               (propertize (concat "@" username) 'face 'bold))
-                      (telega-filter-chats
-                       telega--ordered-chats
-                       '(my-permission :can_invite_users)))))
+                      (telega-filter-chats (telega-chats-list)
+                        '(my-permission :can_invite_users)))))
            (telega-chat--pop-to-buffer chat)
            (telega--sendBotStartMessage
             bot-user chat (plist-get params :startgroup))))
 
         ((plist-get params :start)
          (let* ((bot-chat (telega--searchPublicChat username))
-                (bot-user (telega-chat-user bot-chat 'inc-bots)))
+                (bot-user (telega-chat-user bot-chat)))
            (telega-chat--pop-to-buffer bot-chat)
            (telega--sendBotStartMessage
             bot-user bot-chat (plist-get params :start))))
@@ -167,21 +169,17 @@ PARAMS are additional params."
                   (telega-chat--pop-to-buffer chat)))))
         ))
 
-(defun telega-tme-open-group (group)
-  "Join the GROUP."
-  (let* ((url (concat (or (plist-get telega--options :t_me_url)
-                          "https://t.me/")
-                      "joinchat/" group))
-         (link-check (let ((tl-obj (telega--checkChatInviteLink url)))
-                       (when (telega--tl-error-p tl-obj)
-                         (error "telega: %s" (telega-tl-str tl-obj :error)))
-                       tl-obj))
-         (chat-id (plist-get link-check :chat_id))
-         (chat (when link-check
+(defun telega-tme-open-invite-link (url)
+  "Join the GROUP by invitation link specified by URL."
+  (let* ((invite-link (let ((tl-obj (telega--checkChatInviteLink url)))
+                        (when (telega--tl-error-p tl-obj)
+                          (error "telega: %s" (telega-tl-str tl-obj :error)))
+                        tl-obj))
+         (chat-id (plist-get invite-link :chat_id))
+         (chat (when invite-link
                  (if (zerop chat-id)
-                     ;; Can only join by link
-                     (when (y-or-n-p (format "Join \"%s\"? "
-                                             (telega-tl-str link-check :title)))
+                     ;; Can only join (or request to join) by link
+                     (when (telega-join-invite-link-y-or-n-p invite-link)
                        (telega--joinChatByInviteLink url))
 
                    ;; Can preview messages before deciding to join
@@ -189,19 +187,55 @@ PARAMS are additional params."
     (when chat
       (telega-chat--pop-to-buffer chat))))
 
-(defun telega-tme-open-proxy (_type _proxy)
+(defun telega-tme-open-proxy-tdlib-link (tdlib-link)
+  "Add a proxy defined by internal TDLIB-LINK."
+  (let ((help-window-select t))
+    (with-telega-help-win "*Telega Add Proxy*"
+      (telega-ins-describe-section (telega-i18n "lng_proxy_add"))
+      ;; NOTE: Only MTPROTO proxies may add sponsor channel
+      (when (eq 'proxyTypeMtproto
+                (telega--tl-type (plist-get tdlib-link :type)))
+        (telega-ins--help-message
+         (telega-ins-i18n "lng_proxy_sponsor_warning")))
+      (telega-ins-describe-item "Server"
+        (telega-ins (telega-tl-str tdlib-link :server)))
+      (telega-ins-describe-item "Port"
+        (telega-ins-fmt "%d" (plist-get tdlib-link :port)))
+      (telega-ins--box-button (telega-i18n "lng_proxy_add")
+        'action (lambda-with-current-buffer (_button)
+                  (kill-buffer-and-window)
+                  (telega--addProxy
+                   (list :server (telega-tl-str tdlib-link :server)
+                         :port (plist-get tdlib-link :port)
+                         :type (plist-get tdlib-link :type))
+                   (lambda (_proxy)
+                     (telega-describe-network)))))
+      )))
+
+(defun telega-tme-open-proxy (type proxy)
   "Open the PROXY."
   ;; TYPE is "socks" or "proxy"
   ;; :server, :port, :user, :pass, :secret
-  (message "TODO: `telega-tme-open-proxy'")
+  (message "TODO: `telega-tme-open-proxy' %S / %S" type proxy)
+  (telega-tme-open-proxy-tdlib-link
+   (list :@type "internalLinkTypeProxy"
+         :server (plist-get proxy :server)
+         :port (string-to-number (plist-get proxy :port))
+         :type (cond ((string= type "proxy")
+                      (list :@type "proxyTypeMtproto"
+                            :secret (plist-get proxy :secret)))
+                     ((string= type "socks")
+                      (list :@type "proxyTypeSocks5"
+                            :username (plist-get proxy :user)
+                            :password (plist-get proxy :pass)))
+                     (t
+                      (error "Unknown proxy type: %S" type)))))
   )
 
 (defun telega-tme-open-stickerset (setname)
   "Open sticker set with SETNAME."
-  (let ((sset (telega--searchStickerSet setname)))
-    (unless sset
-      (user-error "No such sticker set: %s" setname))
-    (telega-describe-stickerset sset)))
+  (message "telega: Searching for %S stickerset.." setname)
+  (telega--searchStickerSet setname #'telega-describe-stickerset))
 
 (defun telega-tme-open-theme (_slug)
   (user-error "`telega-tme-open-theme' not yet implemented"))
@@ -216,6 +250,7 @@ PARAMS are additional params."
                                (plist-get lang-pack :native_name)
                                "? "))
       (setq telega-language lang)
+      (telega--setOption :language_pack_id lang 'sync)
       (telega-i18n-init))))
 
 (defun telega-tme-parse-query-string (query-string)
@@ -231,7 +266,7 @@ Multiple params with same name in QUERY-STRING is disallowed."
   "Build a query string for the QUERY-PARAMS.
 QUERY-PARAMS should be in form returned from `telega-tme-parse-query-string'.
 SEMICOLONS and KEEP-EMPTY are passed directly to `url-build-query-string'."
-  
+
   (url-build-query-string
    (telega-plist-map (lambda (key val)
                        (cl-assert (keywordp key))
@@ -255,9 +290,14 @@ Return non-nil, meaning URL has been handled."
              (setq query (telega-plist-del query :domain))
              (apply #'telega-tme-open-username username query)))
           ((string= path "join")
-           (telega-tme-open-group (plist-get query :invite)))
+           (let ((invite-link (concat (or (plist-get telega--options :t_me_url)
+                                          "https://t.me/")
+                                      "joinchat/" (plist-get query :invite))))
+             (telega-tme-open-invite-link invite-link)))
           ((string= path "addstickers")
            (telega-tme-open-stickerset (plist-get query :set)))
+          ((string= path "addemoji")
+           (telega-tme-open-stickerset (plist-get query :set)))
           ((string= path "addtheme")
            (telega-tme-open-theme (plist-get query :slug)))
           ((string= path "setlanguage")
@@ -304,6 +344,7 @@ Matches only t.me, telegram.me and telegram.dog domains.")
 (defun telega-tme-open (url &optional just-convert)
   "Open any URL with https://t.me prefix.
 If JUST-CONVERT is non-nil, return converted link value.
+JUST-CONVERT can be `offline' to avoid calls to telega-server.
 Return non-nil if url has been handled."
   ;; Convert URL to `tg:' form and call `telega-tme-open-tg'
   (when (string-match telega-tme--url-regexp url)
@@ -315,14 +356,12 @@ Return non-nil if url has been handled."
                    (concat "tg:join?invite=" (match-string 1 path)))
                   ((string-match "^/addstickers/\\([a-zA-Z0-9._-]+\\)$" path)
                    (concat "tg:addstickers?set=" (match-string 1 path)))
+                  ((string-match "^/addemoji/\\([a-zA-Z0-9._-]+\\)$" path)
+                   (concat "tg:addemoji?set=" (match-string 1 path)))
                   ((string-match "^/addtheme/\\([a-zA-Z0-9._-]+\\)$" path)
                    (concat "tg:addtheme?slug=" (match-string 1 path)))
-                  ((string-match "^/setlanguage/\\([a-zA-Z0-9._-]+\\)$" path)
-                   (concat "tg:setlanguage?lang=" (match-string 1 path)))
                   ((string-match "^/share/url$" path)
                    (concat "tg:msg_url?" query))
-                  ((string-match "^/\\(socks\\|proxy\\)$" path)
-                   (concat "tg:" (match-string 1 path) "?" query))
                   ((string-match
                     (eval-when-compile
                       (rx (and line-start "/c/"
@@ -333,58 +372,186 @@ Return non-nil if url has been handled."
                    (concat "tg:privatepost?channel=" (match-string 1 path)
                            "&post=" (match-string 2 path)
                            (when query (concat "&" query))))
+                  ;; ((string-match "^/addlist/\\([a-zA-Z0-9._-]+\\)$" path)
+                  ;;  (concat "tg:addlist?slug=" (match-string 1 path)))
                   ((string-match "^/\\+\\([^/]+\\)$" path)
                    (concat "tg:join?invite=" (match-string 1 path)))
-                  ((string-match
-                    (eval-when-compile
-                      (rx (and line-start "/"
-                               (group (1+ (regexp "[a-zA-Z0-9\\.\\_]")))
-                               (? "/" (group (1+ digit))))))
-                    path)
+                  ((string-match "^/\\(socks\\|proxy\\)$" path)
+                   ;; Use getInternalLinkType for proxies
+                   nil)
+
+                  ((and just-convert
+                        (string-match
+                         (eval-when-compile
+                           (rx (and line-start "/"
+                                    (group (1+ (regexp "[a-zA-Z0-9\\.\\_]")))
+                                    (? "/" (group (1+ digit)))
+                                    line-end)))
+                         path))
                    (concat "tg:resolve?domain=" (match-string 1 path)
                            (when (match-string 2 path)
                              (concat "&post=" (match-string 2 path)))
-                           (when query (concat "&" query)))))))
-      (cond (just-convert tg)
+                           (when query (concat "&" query))))
+                  ))
+           (tdlib-link (unless (or tg (eq 'offline just-convert))
+                         (telega--getInternalLinkType url))))
+      (cond (just-convert (or tg tdlib-link))
             (tg (telega-tme-open-tg tg) t)
-            (t
-             (telega-debug "WARN: Can't open \"%s\" internally" url)
-             nil)))))
+            (tdlib-link (telega-tme-open-tdlib-link tdlib-link) t)
+            (t (telega-debug "WARN: Can't open \"%s\" internally" url)
+               nil)))))
 
 (defun telega-tme-open-tdlib-link (tdlib-link)
   "Open TDLib's internal link.
 To convert url to TDLib link, use `telega--getInternalLinkType'."
   (cl-ecase (telega--tl-type tdlib-link)
+    ;; TODO: add other link types
+    (internalLinkTypeUpgradedGift
+     (let ((gift (telega--getUpgradedGift (telega-tl-str tdlib-link :name))))
+       (message "telega: TODO, show upgraded gift \"%s\""
+                (telega-tl-str tdlib-link :name))
+       ))
+
+    (internalLinkTypePublicChat
+     (when-let* ((username (plist-get tdlib-link :chat_username))
+                 (chat (telega--searchPublicChat username)))
+       (telega-chat--pop-to-buffer chat)))
+
     (internalLinkTypeBotStart
      (let* ((bot-username (plist-get tdlib-link :bot_username))
             (bot-chat (telega--searchPublicChat bot-username))
             (bot-user (when bot-chat
-                        (telega-chat-user bot-chat 'inc-bots))))
+                        (telega-chat-user bot-chat))))
        (unless bot-user
          (error "telega: No such bot @%s" bot-username))
 
        (telega-chat--pop-to-buffer bot-chat)
        (setq telega-chatbuf--bot-start-parameter
              (telega-tl-str tdlib-link :start_parameter))
-       (telega-chatbuf--prompt-update)
+       (telega-chatbuf--chat-update "bot-start-parameter")
        ;; Now wait till [START] is pressed
        ))
+
+    ;; (internalLinkTypeAttachmentMenuBot
+    ;;  (let ((url
+    ;;         getAttachmentMenuBot
+    ;;  (telega-browse-url (telega-tl-str tdlib-link :url)))
+
     (internalLinkTypeMessage
-     (telega-tme-open (plist-get tdlib-link :url)))
-
-    (internalLinkTypeActiveSessions
-     )
-
-    (internalLinkTypeVoiceChat
-     )
+     (let* ((msg-info (telega--getMessageLinkInfo
+                       (telega-tl-str tdlib-link :url)))
+            (chat-id (plist-get msg-info :chat_id))
+            (chat (telega-chat-get chat-id))
+            (thread-id (plist-get msg-info :message_thread_id))
+            (msg (plist-get msg-info :message)))
+       (unless msg
+         (error (concat "telega: " (telega-i18n "lng_message_not_found"))))
+       (cond ((telega-zerop thread-id)
+              (telega-msg-goto-highlight msg))
+             ((telega-msg-match-p msg 'is-topic)
+              (let ((topic (telega-topic-by-thread-id chat thread-id)))
+                (telega-topic-goto topic (plist-get msg :id))))
+             (t
+              (telega-chat--goto-thread chat thread-id (plist-get msg :id))))))
 
     (internalLinkTypeChatInvite
-     (let* ((invite-link (plist-get tdlib-link :invite_link))
-            (invite-link-info (telega--checkChatInviteLink invite-link)))
-       ;; TODO
+     (telega-tme-open-invite-link (plist-get tdlib-link :invite_link)))
+
+    (internalLinkTypeStickerSet
+     (telega-tme-open-stickerset (telega-tl-str tdlib-link :sticker_set_name)))
+
+    (internalLinkTypeInstantView
+     (let* ((url (telega-tl-str tdlib-link :url))
+            (fallback-url (telega-tl-str tdlib-link :fallback_url))
+            (iv (telega--getWebPageInstantView url)))
+       (if iv
+           (telega-webpage--instant-view url "Telegra.ph" iv)
+         (telega-browse-url fallback-url 'in-browser))))
+
+    (internalLinkTypeChatBoost
+     (let* ((info (telega--getChatBoostLinkInfo (telega-tl-str tdlib-link :url)))
+            (chat (telega-chat-get (plist-get info :chat_id)))
+            (title (telega-ins--as-string
+                    (telega-ins--msg-sender chat
+                      :with-avatar-p t
+                      :with-username-p 'username
+                      :with-brackets-p t)))
+            (status (telega--getChatBoostStatus chat)))
+       (when (y-or-n-p (concat (telega-i18n "lng_boost_channel_button")
+                               " " title
+                               (format " [%d/%d]"
+                                       (plist-get status :boost_count)
+                                       (plist-get status :next_level_boost_count))
+                               "? "))
+         (telega--boostChat chat)
+         (message "telega: %s" (telega-i18n "lng_boost_channel_you_title"
+                                 :channel title)))))
+
+    (internalLinkTypeLanguagePack
+     (telega-tme-open-lang (telega-tl-str tdlib-link :language_pack_id)))
+
+    (internalLinkTypeStory
+     (let* ((chat (telega--searchPublicChat
+                      (plist-get tdlib-link :story_poster_username)))
+            (story (telega--getStory
+                       (plist-get chat :id) (plist-get tdlib-link :story_id))))
+       (telega-story-open story)))
+
+    (internalLinkTypeWebApp
+     (let* ((chat-bot (telega--searchPublicChat
+                          (plist-get tdlib-link :bot_username)))
+            (_ (unless (telega-chat-bot-p chat-bot)
+                 (error "WebApp username must be bot")))
+            (web-app (telega--searchWebApp
+                         (or (telega-chat-user chat-bot)
+                             (error "WebApp username must be bot"))
+                         (plist-get tdlib-link :web_app_short_name))))
+       (message "found webapp: %S" web-app)))
+
+    (internalLinkTypeAttachmentMenuBot
+     (let ((target-chat (plist-get tdlib-link :target_chat)))
+       (cl-ecase (telega--tl-type target-chat)
+         (targetChatCurrent
+          (let* ((bot-chat (telega--searchPublicChat
+                               (plist-get tdlib-link :bot_username)))
+                 (web-app-info (telega--openWebApp
+                                bot-chat (telega-chat-user bot-chat)
+                                (plist-get tdlib-link :url))))
+            (message "Webapp info: %S" web-app-info)
+            (telega-browse-url (plist-get web-app-info :url))))
+         )))
+
+    (internalLinkTypeProxy
+     (telega-tme-open-proxy-tdlib-link tdlib-link))
+
+    (internalLinkTypeBackground
+     (error "TODO: searchBackground"))
+
+    (internalLinkTypeChatFolderInvite
+     (let ((folder-info (telega--checkChatFolderInviteLink
+                         (plist-get tdlib-link :invite_link))))
+       (error "TODO: telega-describe-chat-folder-info")
        ))
     ))
 
+(defun browse-url-telega (url &rest _args)
+  "Browse Telegram urls with telega."
+  (telega-tme-open url))
+(function-put 'browse-url-telega 'browse-url-browser-kind 'internal)
+
 (provide 'telega-tme)
 
+
+(if (boundp 'browse-url-default-handlers)
+    (add-to-list 'browse-url-default-handlers
+                 (cons telega-tme--url-regexp 'browse-url-telega))
+  ;; Old Emacs
+  (if (and (consp browse-url-browser-function)
+           (not (functionp browse-url-browser-function)))
+      (add-to-list 'browse-url-browser-function
+                   (cons telega-tme--url-regexp 'browse-url-telega))
+    (setq browse-url-browser-function
+          (cons (cons telega-tme--url-regexp 'browse-url-telega)
+                (cons "." browse-url-browser-function)))))
+
 ;;; telega-tme.el ends here
