@@ -89,9 +89,10 @@
           (telega-ins "→")
         (telega-ins "←"))
 
-      (apply 'insert-text-button
-             (telega-user--name (telega-user-get user-id) 'name)
-             (telega-link-props 'user user-id))
+      (telega-ins--raw-button (telega-link-props 'user user-id 'type 'telega)
+        (telega-ins--msg-sender (telega-user-get user-id)
+          :with-avatar-p t
+          :with-username-p t))
       (telega-ins-fmt " %s"
         (substring (plist-get state :@type) 9))
       (cl-case (telega--tl-type state)
@@ -136,7 +137,9 @@ Discard active call if any."
   (when (or force
             (not telega-voip--active-call)
             (y-or-n-p (format "Active call will be discarded, call %s? "
-                              (telega-user--name user 'name))))
+                              (telega-msg-sender-title user
+                                :with-avatar-p t
+                                :with-username-p t))))
     (when telega-voip--active-call
       (telega-voip-discard telega-voip--active-call)
       (setq telega-voip--active-call nil))
@@ -198,7 +201,7 @@ Discard active call if any."
 
 ;;; ellit-org: minor-modes
 ;; ** telega-voip-sounds-mode
-;; 
+;;
 ;; Mode to notify about VoIP call by playing sounds.
 ;; ~telega-voip-sounds-mode~ is enabled by default.  Disable it with:
 ;; #+begin_src emacs-lisp
@@ -231,9 +234,9 @@ If ARG is not given then treat it as 1."
            (when (y-or-n-p "Start already scheduled Voice Chat? ")
              (telega--startScheduledGroupCall group-call)))
           (group-call
-           (user-error "Voice Chat already running"))
+           (user-error "Video Chat already running"))
           (t
-           (telega--createVoiceChat chat
+           (telega--createVideoChat chat
                (read-string
                 (concat (telega-i18n "lng_group_call_edit_title_header")
                         ": ")))))))
@@ -252,7 +255,7 @@ If ARG is not given then treat it as 1."
         (title (read-string
                 (concat (telega-i18n "lng_group_call_edit_title_header")
                         ": "))))
-    (telega--createVoiceChat chat title
+    (telega--createVideoChat chat title
       :start-time start-time)))
 
 (defun telega-video-chat-discard (chat &optional no-confirm-p)
@@ -285,15 +288,6 @@ If ARG is not given then treat it as 1."
       (user-error "Can't stop recording"))
     (telega--endGroupCallRecording group-call)))
 
-(defun telega-video-chat-toggle-footer (chat)
-  "Toggle video chat visibility in CHAT's footer/modeline."
-  (interactive (list (or telega-chatbuf--chat (telega-chat-at (point)))))
-  (with-telega-chatbuf chat
-    (setq telega-chatbuf--video-chat-hidden
-          (not telega-chatbuf--video-chat-hidden))
-    (telega-chatbuf--footer-update)
-    (telega-chatbuf--modeline-update)))
-
 (defun telega-group-call--ensure (group-call)
   "Ensure GROUP-CALL is in the `telega--group-calls'.
 Return GROUP-CALL."
@@ -322,9 +316,9 @@ Return GROUP-CALL."
 
 (defun telega-group-call-get-chat (group-call-id)
   "Return chat with group call of GROUP-CALL-ID."
-  (cl-find group-call-id
-           telega--ordered-chats
-           :key (telega--tl-prop :video_chat :group_call_id)))
+  (telega-chat-by (lambda (chat)
+                    (eq group-call-id
+                        (telega--tl-get chat :video_chat :group_call_id)))))
 
 (defun telega-group-call--participant-svg-outline (svg circle
                                                        &optional speaking-p)
@@ -334,7 +328,9 @@ SPEAKING-P if participant is speaking."
               :stroke-width (/ (nth 2 circle) (if speaking-p 5 10))
               :stroke-color (telega-color-name-as-hex-2digits
                              (face-foreground
-                              (if speaking-p 'font-lock-string-face 'shadow)
+                              (if speaking-p
+                                  'font-lock-string-face
+                                'telega-shadow)
                               nil t))
               :opacity 0.85
               :fill-opacity "0"))
@@ -383,35 +379,49 @@ CHEIGHT is height in chars, default is 1."
   "Set GROUP-CALL's title to TITLE.
 If TITLE is not specified, ask user interactively for the new title."
   (unless title
-    (setq title (read-string "Group Call Title: ")))
+    (setq title (read-string
+                 (concat (telega-i18n "lng_group_call_edit_title_header")
+                         ": ")
+                 (telega-tl-str group-call :title))))
   (telega--setGroupCallTitle group-call title))
 
 (defun telega-describe-group-call--inserter (group-call-id)
   "Inserter for the voice chat."
-  (let* ((group-call (telega-group-call-get group-call-id))
-         (can-manage-p (plist-get group-call :can_be_managed)))
-    (telega-ins (telega-i18n "lng_group_call_title") ": "
-                (or (telega-tl-str group-call :title)
-                    (propertize "No title" 'face 'shadow)))
-    (telega-ins " ")
+  (let ((group-call (telega-group-call-get group-call-id))
+        (chat (telega-group-call-get-chat group-call-id)))
+    ;; Buttons section
     (if (plist-get group-call :is_joined)
-        (telega-ins--button "Leave"
+        (telega-ins--box-button (telega-i18n "lng_group_call_leave")
           :value group-call
           :action #'telega-group-call-leave)
-      (telega-ins--button "Join"
+      (telega-ins--box-button (telega-i18n "lng_group_call_join")
         :value group-call
         :action #'telega-group-call-join))
-
-    (when can-manage-p
+    (when (plist-get group-call :can_be_managed)
       (telega-ins " ")
-      (telega-ins--button (telega-i18n "lng_group_call_end")
+      (telega-ins--box-button (telega-i18n "lng_group_call_end")
         :value group-call
         :action #'telega--endGroupCall)
       (telega-ins " ")
-      (telega-ins--button "Set Title"
+      (telega-ins--box-button (telega-i18n "lng_group_call_edit_title")
         :value group-call
         :action #'telega-group-call-set-title))
 
+    (telega-ins "\n")
+    (telega-ins-describe-item "Id"
+      (telega-ins-fmt "%d" group-call-id))
+    (telega-ins-describe-item (telega-i18n "lng_group_call_edit_title_header")
+      (telega-ins (telega-tl-str group-call :title)))
+
+    (when (and (plist-get group-call :is_rtmp_stream)
+               (telega-chat-match-p chat 'me-is-owner))
+      (telega-ins-describe-item "RTMP URL"
+        (telega--getVideoChatRtmpUrl chat
+          (telega--gen-ins-continuation-callback 'loading
+            (lambda (rtmp-url)
+              (telega-ins (plist-get rtmp-url :url)
+                          (plist-get rtmp-url :stream_key)))))))
+
     (telega-ins "\n")
     ))
 
