@@ -1,19 +1,19 @@
 ;;; telega.el --- Telegram client (unofficial)  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2016-2021 by Zajcev Evgeny
+;; Copyright (C) 2016-2025 by Zajcev Evgeny
 ;; Copyright (C) 2019-2020 by Brett Gilio
 
 ;; Author: Zajcev Evgeny <zevlg@yandex.ru>
 ;; Created: Wed Nov 30 19:04:26 2016
 ;; Keywords: comm
-;; Package-Requires: ((emacs "26.1") (visual-fill-column "1.9") (rainbow-identifiers "0.2.2"))
+;; Package-Requires: ((emacs "27.1") (visual-fill-column "1.9") (transient "0.3.0"))
 ;; URL: https://github.com/zevlg/telega.el
-;; Package-Version: 0.8.3
-;; Package-Revision: ac3634e2e7ef
-(defconst telega-version "0.8.03")
+;; Package-Version: 20251023.1131
+;; Package-Revision: c8000bb5a954
+(defconst telega-version "0.8.554")
 (defconst telega-server-min-version "0.7.7")
-(defconst telega-tdlib-min-version "1.8.0")
-(defconst telega-tdlib-max-version "1.8.0")
+(defconst telega-tdlib-min-version "1.8.55")
+(defconst telega-tdlib-max-version "1.8.55")
 
 (defconst telega-tdlib-releases '("1.8.0" . "1.9.0")
   "Cons cell with current and next TDLib releases.
@@ -48,6 +48,7 @@ Used for manual generation.")
 (require 'telega-server)
 (require 'telega-root)
 (require 'telega-ins)
+(require 'telega-match)
 (require 'telega-filter)
 (require 'telega-chat)
 (require 'telega-folders)
@@ -61,6 +62,7 @@ Used for manual generation.")
 (require 'telega-notifications)
 (require 'telega-modes)
 (require 'telega-i18n)
+(require 'telega-story)
 (require 'telega-tdlib)
 (require 'telega-tdlib-events)
 
@@ -112,7 +114,7 @@ Used for manual generation.")
     ;;   {{{fundoc(telega-switch-unread-chat, 2)}}}
     ;;
     ;;   Customizable options:
-    ;;   - {{{user-option(telega-filter-unread-chats, 4)}}}
+    ;;   - {{{user-option(telega-unread-chat-temex, 4)}}}
     (define-key map (kbd "u") 'telega-switch-unread-chat)
     ;;; ellit-org: prefix-map-bindings
     ;; - {{{where-is(telega-browse-url,telega-prefix-map)}}} ::
@@ -151,12 +153,11 @@ can't write to `telega-server-logfile'" logfile-dir)))
   (interactive
    (list (if (not telega-accounts)
              (user-error "telega: Single account setup, see `telega-accounts'")
-           (funcall telega-completing-read-function
-                    "Telegram Account: "
-                    (mapcar #'car
-                            (cl-remove-if #'telega-account--current-p
-                                          telega-accounts))
-                    nil 'require-match))))
+           (telega-completing-read
+            "Telegram Account: "
+            (mapcar #'car (cl-remove-if #'telega-account--current-p
+                                        telega-accounts))
+            nil 'require-match))))
 
   (let ((account (assoc account-name telega-accounts)))
     (cl-assert account)
@@ -167,7 +168,7 @@ can't write to `telega-server-logfile'" logfile-dir)))
       ;; After setting all the variables of the account it must
       ;; become current
       (unless (telega-account--current-p account)
-        (user-error "telega: Invalid config for \"%s\" account: at least 'telega-database-dir variable must be provided"))
+        (user-error "telega: Invalid config for \"%s\" account: at least 'telega-database-dir variable must be provided" account))
 
       (telega-server-kill)
       ;; Wait for server to die
@@ -176,6 +177,25 @@ can't write to `telega-server-logfile'" logfile-dir)))
 
     (telega nil)))
 
+(defun telega-set-network-type (network-type)
+  "Interactively change network type.
+Can be called even if telega is not running.
+Modifies `telega-network-type' by side-effect."
+  (interactive
+   (let* ((choices (mapcar (lambda (nt)
+                             (list (substring (plist-get (cdr nt) :@type) 11)
+                                   (car nt)))
+                           telega-tdlib-network-type-alist))
+          (choice (telega-completing-read "Set Network Type: " choices nil t))
+          (network-type (car (alist-get choice choices nil nil 'string=))))
+     (list network-type)))
+
+  (setq telega-network-type network-type)
+  (when (telega-server-live-p)
+    (let ((tdlib-network-type
+           (alist-get network-type telega-tdlib-network-type-alist)))
+      (telega--setNetworkType tdlib-network-type))))
+
 ;;;###autoload
 (defun telega (&optional arg)
   "Start telega.el Telegram client.
@@ -197,12 +217,26 @@ If `\\[universal-argument]' is specified, then do not pop to root buffer."
 
       (telega--create-hier)
       (telega--init-vars)
+      ;; NOTE: rootbuf might use i18n strings, so initialize it with
+      ;; default strings before entering rootbuf
+      (let ((telega-language "en"))
+        (telega-i18n-init))
       (with-current-buffer (get-buffer-create telega-root-buffer-name)
         (telega-root-mode))
 
       (telega-server--ensure-build)
       (telega-server--start)
-      (telega-i18n-init))
+      (telega-i18n-init)
+
+      ;; Some options are better to set before calling setTdlibParameters
+      ;; ref: https://t.me/tdlibchat/136047
+      (telega--tl-dolist ((prop-name value) telega-options-plist)
+        (telega--setOption prop-name value))
+
+      ;; For telega-auto-download-mode and network statistics
+      (unless (eq 'other telega-network-type)
+        (telega-set-network-type telega-network-type))
+      )
 
     (unless arg
       (pop-to-buffer-same-window telega-root-buffer-name))))
@@ -228,57 +262,41 @@ Works only if current state is `authorizationStateWaitCode'."
 
 (defun telega--authorization-ready ()
   "Called when tdlib is ready to receive queries."
-  ;; Validate tdlib version
-  (let ((version-error-msg
-         (cond ((version< (plist-get telega--options :version)
-                          telega-tdlib-min-version)
-                (format "TDLib version=%s < %s (min required), \
-please upgrade TDLib and recompile `telega-server'"
-                        (plist-get telega--options :version)
-                        telega-tdlib-min-version))
-               ((and telega-tdlib-max-version
-                     (version< telega-tdlib-max-version
-                               (plist-get telega--options :version)))
-                (format "TDLib version=%s > %s (max required), \
-please downgrade TDLib and recompile `telega-server'"
-                        (plist-get telega--options :version)
-                        telega-tdlib-max-version)))))
-    (if version-error-msg
-        (progn
-          (telega-kill 'force)
-          (run-with-timer 0 nil #'warn version-error-msg))
+  ;; In case language pack id has not yet been selected, then select
+  ;; suggested one or fallback to "en"
+  (unless (plist-get telega--options :language_pack_id)
+    (telega--setOption :language_pack_id
+      (or (plist-get telega--options :suggested_language_pack_id) "en")))
 
-      ;; Versions are ok
-      (telega--setOptions telega-options-plist)
-      ;; In case language pack id has not yet been selected, then select
-      ;; suggested one or fallback to "en"
-      (unless (plist-get telega--options :language_pack_id)
-        (telega--setOption :language_pack_id
-          (or (plist-get telega--options :suggested_language_pack_id) "en")))
+  ;; Apply&update notifications settings
+  (dolist (scope-type telega-notification-scope-types)
+    (when-let ((settings
+                (alist-get (car scope-type) telega-notifications-defaults)))
+      (apply #'telega--setScopeNotificationSettings (cdr scope-type) settings)))
 
-      ;; Apply&update notifications settings
-      (dolist (scope-type telega-notification-scope-types)
-        (when-let ((settings
-                    (alist-get (car scope-type) telega-notifications-defaults)))
-          (apply #'telega--setScopeNotificationSettings (cdr scope-type) settings)))
+  ;; Fetch blocked users for all block lists
+  (telega--getBlockedMessageSenders 'blockListMain 0
+    #'telega--on-blocked-senders-load)
+  (telega--getBlockedMessageSenders 'blockListStories 0
+    #'telega--on-blocked-senders-load)
 
-      ;; Fetch blocked users
-      (telega--getBlockedMessageSenders 0 #'telega--on-blocked-senders-load)
+  ;; NOTE: telega--scope-notification-alist will be updated upon
+  ;; `updateScopeNotificationSettings' event
 
-      ;; NOTE: telega--scope-notification-alist will be updated upon
-      ;; `updateScopeNotificationSettings' event
+  ;; All OK, request for chats/users/etc
+  (telega-status--set nil "Loading chats...")
 
-      ;; All OK, request for chats/users/etc
-      (telega-status--set nil "Loading chats...")
+  (telega--loadChats (list :@type "chatListMain")
+    #'telega--on-initial-chats-load)
+  ;; NOTE: We hope `telega--getChats' will return all chats in the
+  ;; Archive, in general this is not true, we need special callback to
+  ;; continue fetching, as with "chatListMain" list
+  (telega--loadChats (list :@type "chatListArchive"))
 
-      (telega--loadChats (list :@type "chatListMain")
-        #'telega--on-initial-chats-load)
-      ;; NOTE: We hope `telega--getChats' will return all chats in the
-      ;; Archive, in general this is not true, we need special callback to
-      ;; continue fetching, as with "chatListMain" list
-      (telega--loadChats (list :@type "chatListArchive"))
+  ;; Load general tags for saved messages
+  (telega--getSavedMessagesTags 0 #'telega-saved-messages-tags-ensure)
 
-      (run-hooks 'telega-ready-hook))))
+  (run-hooks 'telega-ready-hook))
 
 ;;;###autoload
 (defun telega-version (&optional insert-p)
@@ -287,17 +305,22 @@ If `\\[universal-argument]' is specified, then insert the version
 string at point."
   (interactive "P")
   (let* ((tdlib-version (plist-get telega--options :version))
+         (tdlib-sha1 (or (plist-get telega--options :commit_hash)
+                         "unknown"))
          (version (concat "telega v"
                           telega-version
                           " ("
                           (if tdlib-version
-                              (concat "TDLib v" tdlib-version)
+                              (concat "TDLib v" tdlib-version "-"
+                                      (substring tdlib-sha1 0 7))
                             "TDLib version unknown. Make sure server is running")
                           ")"
                           " (telega-server v"
                           (telega-server-version)
                           (when telega-use-docker
-                            (concat " [docker]"))
+                            (format " [%s]" (if (stringp telega-use-docker)
+                                                telega-use-docker
+                                              "docker")))
                           ")")))
     (if insert-p
         (insert version)
@@ -320,6 +343,18 @@ string at point."
                            "unknown")
               "\n")
       (insert "*Emacs*: " (let (emacs-build-time) (emacs-version)) "\n")
+      (telega-ins "*Features*: "
+                  (when (image-type-available-p 'imagemagick)
+                    "imagemagick ")
+                  (when (image-type-available-p 'svg)
+                    "svg ")
+                  (when (image-type-available-p 'webp)
+                    "webp ")
+                  (when (executable-find "ffmpeg")
+                    "ffmpeg ")
+                  (when (executable-find "tgs2png")
+                    "tgs2png ")
+                  "\n")
       (insert "*Telega*: " (telega-version) "\n")
       (when-let ((melpa-pkg (ignore-errors
                               (read (find-file-noselect
@@ -357,19 +392,28 @@ string at point."
 ;; Enable some global minor modes by default
 (telega-patrons-mode 1)
 (telega-active-locations-mode 1)
+(telega-active-video-chats-mode 1)
+(telega-contact-birthdays-mode 1)
+(telega-active-stories-mode 1)
+(telega-network-stats-mode 1)
 
 ;; Enable root auto fill mode by default
 (telega-root-auto-fill-mode 1)
 
-(run-hooks 'telega-load-hook)
-;; Load hook might install new symbols into
-;; `telega-symbol-widths'
-(telega-symbol-widths-install telega-symbol-widths)
+;; Enable chat auto fill mode by default in the chatbufs
+(add-hook 'telega-chat-mode-hook #'telega-chat-auto-fill-mode)
 
 ;; For messages loaded from history
-(add-hook 'telega-chat-insert-message-hook #'telega-msg-run-ignore-predicates)
+(add-hook 'telega-chatbuf-pre-msg-insert-hook #'telega-msg-run-ignore-predicates)
+(add-hook 'telega-chatbuf-pre-msg-insert-hook #'telega-msg--replied-message-fetch)
+(add-hook 'telega-chatbuf-pre-msg-insert-hook #'telega-msg--replied-story-fetch)
+(add-hook 'telega-chatbuf-pre-msg-insert-hook #'telega-msg--story-fetch)
+(add-hook 'telega-chatbuf-pre-msg-insert-hook #'telega-msg--custom-emojis-fetch)
+(add-hook 'telega-chatbuf-post-msg-update-hook #'telega-msg--custom-emojis-fetch)
 
 ;; WARN about usage of the obsolete variables
 (require 'telega-obsolete)
 
+(run-hooks 'telega-load-hook)
+
 ;;; telega.el ends here
